@@ -20,6 +20,7 @@ module Data.ByteString.Base64.Internal
     , joinWith
     , done
     , peek8, poke8, peek8_32
+    , reChunkIn
     ) where
 
 import Data.Bits ((.|.), (.&.), shiftL, shiftR)
@@ -111,6 +112,7 @@ mkEncodeTable alphabet@(PS afp _ _) =
 -- > joinWith "|" 2 "----" = "--|--|"
 --
 -- > joinWith "\r\n" 3 "foobarbaz" = "foo\r\nbar\r\nbaz\r\n"
+-- > joinWith "x" 3 "fo" = "fox"
 joinWith :: ByteString  -- ^ String to intersperse and end with
          -> Int         -- ^ Interval at which to intersperse, in bytes
          -> ByteString  -- ^ String to transform
@@ -124,21 +126,28 @@ joinWith brk@(PS bfp boff blen) every bs@(PS sfp soff slen)
     withForeignPtr bfp $ \bptr -> do
       withForeignPtr sfp $ \sptr -> do
           let bp = bptr `plusPtr` boff
-          let dEnd = dptr `plusPtr` dlen
-              loop !dp !sp | dp >= dEnd = return ()
-                           | otherwise = do
-                let n = min every (dEnd `minusPtr` dp)
-                memcpy dp sp (fromIntegral n)
-                memcpy (dp `plusPtr` n) bp (fromIntegral blen)
-                loop (dp `plusPtr` (n + blen)) (sp `plusPtr` every)
-          loop dptr (sptr `plusPtr` soff)
-  where dlen = slen + blen * numBreaks
+              sp0 = sptr `plusPtr` soff
+              sLast = sp0 `plusPtr` (every * numBreaks)
+              loop !dp !sp
+                  | sp == sLast = do
+                      let n = sp0 `plusPtr` slen `minusPtr` sp
+                      memcpy dp sp (fromIntegral n)
+                      memcpy (dp `plusPtr` n) bp (fromIntegral blen)
+                  | otherwise = do
+                memcpy dp sp (fromIntegral every)
+                let dp' = dp `plusPtr` every
+                memcpy dp' bp (fromIntegral blen)
+                loop (dp' `plusPtr` blen) (sp `plusPtr` every)
+          loop dptr sp0
+  where dlast = slen + blen * numBreaks
+        dlen | slen `mod` every > 0 = dlast + blen
+             | otherwise            = dlast
         numBreaks = slen `div` every
 
 -- | Decode a base64-encoded string.  This function strictly follows
 -- the specification in RFC 4648,
 -- <http://www.apps.ietf.org/rfc/rfc4648.html>.
--- This function takes the decoding table (for @base64@ or @base64url@) as 
+-- This function takes the decoding table (for @base64@ or @base64url@) as
 -- the first paramert.
 decodeWithTable :: ForeignPtr Word8 -> ByteString -> Either String ByteString
 decodeWithTable decodeFP (PS sfp soff slen)
@@ -195,7 +204,7 @@ data D = D { dNext :: {-# UNPACK #-} !(Ptr Word8)
 -- following the specification from RFC 4648,
 -- <http://www.apps.ietf.org/rfc/rfc4648.html>, and will not generate
 -- parse errors no matter how poor its input.
--- This function takes the decoding table (for @base64@ or @base64url@) as 
+-- This function takes the decoding table (for @base64@ or @base64url@) as
 -- the first paramert.
 decodeLenientWithTable :: ForeignPtr Word8 -> ByteString -> ByteString
 decodeLenientWithTable decodeFP (PS sfp soff slen)
@@ -250,3 +259,27 @@ x = 255
 done :: Integral a => a
 done = 99
 {-# INLINE done #-}
+
+-- This takes a list of ByteStrings, and returns a list in which each
+-- (apart from possibly the last) has length that is a multiple of n
+reChunkIn :: Int -> [ByteString] -> [ByteString]
+reChunkIn _ [] = []
+reChunkIn _ [y] = [y]
+reChunkIn n (y : ys) = case B.length y `divMod` n of
+                       (_, 0) ->
+                           y : reChunkIn n ys
+                       (d, _) ->
+                           case B.splitAt (d * n) y of
+                           (prefix, suffix) -> prefix : fixup suffix ys
+    where fixup acc [] = [acc]
+          fixup acc (z : zs) = case B.splitAt (n - B.length acc) z of
+                               (prefix, suffix) ->
+                                   let acc' = acc `B.append` prefix
+                                   in if B.length acc' == n
+                                      then let zs' = if B.null suffix
+                                                     then          zs
+                                                     else suffix : zs
+                                           in acc' : reChunkIn n zs'
+                                      else -- suffix must be null
+                                           fixup acc' zs
+
